#!/usr/bin/env python
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import jujuresources


def status_set(status, message):
    import subprocess
    try:
        subprocess.check_call(['status-set', status, message])
    except (subprocess.CalledProcessError, OSError):
        jujuresources.juju_log('status: {}: {}'.format(status, message), 'ERROR')


def bootstrap_resources():
    """
    Install required resources defined in resources.yaml
    """
    mirror_url = jujuresources.config_get('resources_mirror')
    if not jujuresources.fetch(mirror_url=mirror_url):
        status_set('blocked', 'Resources unavailable; manual intervention required')
        return False
    jujuresources.install(['pathlib', 'pyaml', 'six', 'charmhelpers', 'jujubigdata'])
    return True


def manage():
    if not bootstrap_resources():
        # defer until resources are available, since charmhelpers, and thus
        # the framework, (will be) managed by jujuresources
        return

    from charmhelpers.core import charmframework
    import jujubigdata
    import callbacks  # noqa  (ignore when linting)

    # list of keys required to be in the dist.yaml
    master_reqs = ['vendor', 'hadoop_version', 'packages', 'groups', 'users',
                   'dirs', 'ports']
    dist_config = jujubigdata.utils.DistConfig(filename='dist.yaml',
                                               required_keys=master_reqs)
    hadoop = jujubigdata.handlers.HadoopBase(dist_config)
    hdfs = jujubigdata.handlers.HDFS(hadoop)
    hdfs_port = dist_config.port('namenode')
    webhdfs_port = dist_config.port('nn_webapp_http')
    secondary = jujubigdata.relations.NameNodeMaster(spec=hadoop.spec, port=hdfs_port, webhdfs_port=webhdfs_port,
                                                     relation_name='secondary')
    datanodes = jujubigdata.relations.NameNodeMaster(spec=hadoop.spec, port=hdfs_port, webhdfs_port=webhdfs_port)
    clients = jujubigdata.relations.NameNode(spec=hadoop.spec, port=hdfs_port, webhdfs_port=webhdfs_port)
    manager = charmframework.Manager([
        {
            'name': 'hadoop-base',
            'requires': [
                hadoop.verify_conditional_resources,
            ],
            'callbacks': [
                hadoop.install,
            ],
        },
        {
            'name': 'hdfs-master',
            'provides': [
                secondary,
                datanodes,
                clients,
            ],
            'requires': [
                hadoop.is_installed,
                jujubigdata.relations.DataNode(optional=True),
                jujubigdata.relations.SecondaryNameNode(optional=True),
            ],
            'callbacks': [
                datanodes.register_host_ips,
                secondary.register_host_ips,
                jujubigdata.utils.update_etc_hosts_from_kv,
                hdfs.configure_namenode,
                hdfs.register_slaves,
                hdfs.format_namenode,
                hdfs.start_namenode,
                hdfs.create_hdfs_dirs,
                charmframework.helpers.open_ports(dist_config.exposed_ports('hdfs-master')),
            ],
            'cleanup': [
                charmframework.helpers.close_ports(dist_config.exposed_ports('hdfs-master')),
                hdfs.stop_namenode,
            ],
        },
    ])
    manager.manage()


if __name__ == '__main__':
    manage()
